@@ -481,18 +481,6 @@ XMLSEC_EXPORT void                      xmlSecTransformDebugXmlDump(xmlSecTransf
          (( transform )->id->objSize >= sizeof(xmlSecTransform)) && \
          (( transform )->id->name != NULL))
 
-/**
- * xmlSecTransformCheckType:
- * @transform:          the pointer to transform.
- * @t:                  the transform type.
- *
- * Macro. Returns 1 if the @transform is valid and has specified type @t
- * or 0 otherwise.
- */
-#define xmlSecTransformCheckType(transform, t) \
-        (xmlSecTransformIsValid(( transform )) && \
-        ((( transform )->id->type) == ( t )))
-
 /**
  * xmlSecTransformCheckId:
  * @transform:          the pointer to transform.
@@ -962,6 +950,8 @@ XMLSEC_EXPORT int               xmlSecTransformXPointerSetExpr          (xmlSecT
                                                                          xmlSecNodeSetType nodeSetType,
                                                                          xmlNodePtr hereNode);
 #ifndef XMLSEC_NO_XSLT
+#include <libxslt/security.h>
+
 /**
  * xmlSecTransformXsltId:
  *
@@ -970,6 +960,7 @@ XMLSEC_EXPORT int               xmlSecTransformXPointerSetExpr          (xmlSecT
 #define xmlSecTransformXsltId \
         xmlSecTransformXsltGetKlass()
 XMLSEC_EXPORT xmlSecTransformId xmlSecTransformXsltGetKlass             (void);
+XMLSEC_EXPORT void              xmlSecTransformXsltSetDefaultSecurityPrefs(xsltSecurityPrefsPtr sec);
 #endif /* XMLSEC_NO_XSLT */
 
 /**
