@@ -35,10 +35,6 @@
 #include <xmlsec/mscrypto/x509.h>
 #include "private.h"
 
-#if defined(__MINGW32__)
-#  include "xmlsec-mingw.h"
-#endif
-
 /**************************************************************************
  *
  * Internal MSCRYPTO X509 store CTX
